@@ -1,14 +1,13 @@
-"""This module provides the base types used by :mod:`virtualbox.library`.
-"""
+"""This module provides the base types used by :mod:`virtualbox.library`."""
 
 import re
 import inspect
-import sys
 import platform
+import time
 
-# Py2 and Py3 compatibility  
+# Py2 and Py3 compatibility
 try:
-    import __builtin__ as builtin 
+    import __builtin__ as builtin
 except:
     import builtins as builtin
 
@@ -54,10 +53,11 @@ def add_metaclass(metaclass):
     return wrapper
 
 
-@add_metaclass(EnumType)   
+@add_metaclass(EnumType)
 class Enum(object):
     """Enum objects provide a container for VirtualBox enumerations"""
     _enums = {}
+
     def __init__(self, value):
         if value not in self._lookup_label:
             raise ValueError("Can not find enumeration where value=%s" % value)
@@ -79,7 +79,7 @@ class Enum(object):
         return self.__cmp__(k) == 0
 
     def __cmp__(self, k):
-        return cmp(int(self), int(k))
+        return (int(self) > int(k)) - (int(self) < int(k))
 
     def __getitem__(self, k):
         return self.__class__[k]
@@ -96,11 +96,12 @@ class VBoxErrorMeta(type):
 
 
 @add_metaclass(VBoxErrorMeta)
-class VBoxError(Exception): 
+class VBoxError(Exception):
     """Generic VBoxError"""
     name = "undef"
     value = -1
     msg = ""
+
     def __str__(self):
         return "0x%x (%s)" % (self.value, self.msg)
 
@@ -109,7 +110,7 @@ class Interface(object):
     """Interface objects provide a wrapper for the VirtualBox COM objects"""
     def __init__(self, interface=None):
         if isinstance(interface, Interface):
-            import virtualbox 
+            import virtualbox
             manager = virtualbox.Manager()
             self._i = manager.cast_object(interface, self.__class__)._i
         else:
@@ -132,13 +133,23 @@ class Interface(object):
             return cast_to_valuetype(value)
 
     def _search_attr(self, name, prefix=None):
-        attr_name = name
-        attr = getattr(self._i, attr_name, None)
-        # if a prefix is defined, try to get that prefixed name and use that
-        # attribute instead, else, stick with the attr value pulled out above
+        attr_names = [name]
         if prefix is not None:
-            prefix_name = prefix + name[0].upper() + name[1:]
-            attr = getattr(self._i, prefix_name, attr)
+            attr_names.append(prefix + name[0].upper() + name[1:])
+        # Sometimes xpcom interface fails to return the attribute.  Check a few
+        # times before giving up.
+        for i in range(3):
+            for attr_name in attr_names:
+                attr = getattr(self._i, attr_name, self)
+                if attr is not self:
+                    break
+            else:
+                # Failed to get attribute, do a quick sleep and try again.
+                time.sleep(0.1)
+                continue
+            break
+        else:
+            raise AttributeError("Failed to find attribute %s in %s" % (name, self))
         return attr
 
     def _get_attr(self, name):
@@ -153,9 +164,13 @@ class Interface(object):
         if inspect.isfunction(attr) or inspect.ismethod(attr):
             return self._call_method(attr, value)
         else:
+            if isinstance(value, Enum):
+                value = int(value)
             return setattr(self._i, name, value)
 
-    def _call(self, name, in_p=[]):
+    def _call(self, name, in_p=None):
+        if in_p is None:
+            in_p = []
         global vbox_error
         method = self._search_attr(name)
         if inspect.isfunction(method) or inspect.ismethod(method):
@@ -163,30 +178,28 @@ class Interface(object):
         else:
             return method
 
-    def _call_method(self, method, in_p=[]):
+    def _call_method(self, method, in_p=None):
+        if in_p is None:
+            in_p = []
         in_params = [self._cast_to_valuetype(p) for p in in_p]
         try:
             ret = method(*in_params)
         except Exception as exc:
             errno = getattr(exc, 'errno', getattr(exc, 'hresult', -1))
-            errno = errno & 0xFFFFFFFF
+            errno &= 0xFFFFFFFF
             errclass = vbox_error.get(errno, VBoxError)
             errobj = errclass()
             errobj.exc = exc
             errobj.value = errno
-            # TODO: Is this the only way to get a message from exc... 
+            # TODO: Is this the only way to get a message from exc...
             #       does this also vary between nix vs windows.
             errobj.msg = None
             if platform.system() == 'Windows':
                 if hasattr(exc, 'args'):
-                    print(exc.args)
                     errobj.msg = exc.args[2][2]
-	    # TODO: get the Linux/Darwin specific args struct
+            # TODO: get the Linux/Darwin specific args struct
 
             if errobj.msg is None:
                 errobj.msg = getattr(exc, 'msg', getattr(exc, 'message'))
             raise errobj
         return ret
-
-
-
