@@ -2,12 +2,12 @@
 =======================
 
 The :py:class:`MachinePool` manages a pool of linked clones against a defined
-"root machine".  This module works with multiple processes running on the 
+"root machine".  This module works with multiple processes running on the
 host machine at a time.  It manages a resource lock over the root virtual
-machine to ensure consistency. 
+machine to ensure consistency.
 
 In this example the machine *win7* has a current version of guest editions
-installed and is in a powered off state. 
+installed and is in a powered off state.
 
 
 Create multiple clones::
@@ -16,7 +16,7 @@ Create multiple clones::
     sessions = []
     for i in range(3):
         sessions.append(pool.acquire("Mick", "password"))
-    
+
     # You now have three running machines.
     for session in sessions:
         with session.guest.create_session("Mick", "password") as gs:
@@ -34,7 +34,7 @@ A reliable version of the above code would look like this::
     try:
         for i in range(3):
             sessions.append(pool.acquire("Mick", "password"))
-        
+
         # You now have three running machines.
         for session in sessions:
             with session.guest.create_session("Mick", "password") as gs:
@@ -48,24 +48,24 @@ A reliable version of the above code would look like this::
             except Exception as err:
                 print("Error raised on release: %s" % err)
 
-
 """
+
 from __future__ import absolute_import
 from contextlib import contextmanager
 import time
 
 from virtualbox import VirtualBox
-from virtualbox import Session 
+from virtualbox import Session
 from virtualbox.library import LockType
-from virtualbox.library import SessionState 
-from virtualbox.library import MachineState 
-from virtualbox.library import DeviceType 
-from virtualbox.library import DeviceActivity 
+from virtualbox.library import SessionState
+from virtualbox.library import DeviceType
+from virtualbox.library import DeviceActivity
+from virtualbox.library import OleErrorUnexpected
+
 
 class MachinePool(object):
-    """MachinePool manages a pool of resources and enable cross process 
-    coordination of a linked machine clone. 
-    """
+    """MachinePool manages a pool of resources and enable cross process
+    coordination of a linked machine clone. """
     def __init__(self, machine_name):
         """Create a MachinePool instance.
 
@@ -74,14 +74,16 @@ class MachinePool(object):
         """
         self.machine_name = machine_name
         with self._lock() as session:
-            if not session.machine.current_snapshot:
-                console = session.console
-                p = console.take_snapshot('initialised', 'root machine')
-                p.wait_for_completion(60*1000)
+            machine = session.machine
+            if not machine.current_snapshot:
+                p, id_p = machine.take_snapshot('initialised',
+                                                'root machine',
+                                                False)
+                p.wait_for_completion(60 * 1000)
 
     @contextmanager
     def _lock(self, timeout_ms=-1):
-        "Exclusive lock over root machine"
+        """Exclusive lock over root machine"""
         vbox = VirtualBox()
         machine = vbox.find_machine(self.machine_name)
         wait_time = 0
@@ -97,13 +99,13 @@ class MachinePool(object):
             else:
                 try:
                     yield session
-                finally: 
+                finally:
                     session.unlock_machine()
                 break
 
     @property
     def _clones(self):
-        "Yield all machines under this pool"
+        """Yield all machines under this pool"""
         vbox = VirtualBox()
         machines = []
         for machine in vbox.machines:
@@ -118,22 +120,26 @@ class MachinePool(object):
         clone = vbox.find_machine(session.machine.name)
         try:
             p = session.console.power_down()
-            p.wait_for_completion(60*1000)
-            session.unlock_machine()
+            p.wait_for_completion(60 * 1000)
+            try:
+                session.unlock_machine()
+            except OleErrorUnexpected:
+                # session seems to become unlocked automatically after
+                # wait_for_completion is called after the power_down?
+                pass
             session = clone.create_session()
-            console = session.console
-            p = console.restore_snapshot()
-            p.wait_for_completion(60*1000)
+            p = session.machine.restore_snapshot()
+            p.wait_for_completion(60 * 1000)
             return clone
         finally:
             if session.state == SessionState.locked:
                 session.unlock_machine()
 
     def acquire(self, username, password, frontend='headless'):
-        "Acquire a Machine resource."
+        """Acquire a Machine resource."""
         with self._lock() as root_session:
             for clone in self._clones:
-                # Search for a free clone 
+                # Search for a free clone
                 session = Session()
                 try:
                     clone.lock_machine(session, LockType.write)
@@ -141,8 +147,8 @@ class MachinePool(object):
                     continue
                 else:
                     try:
-                        p = session.console.restore_snapshot()
-                        p.wait_for_completion(60*1000)
+                        p = session.machine.restore_snapshot()
+                        p.wait_for_completion(60 * 1000)
                     except:
                         pass
                     session.unlock_machine()
@@ -152,25 +158,27 @@ class MachinePool(object):
                 machine = root_session.machine
                 clone = machine.clone(name="%s Pool" % self.machine_name)
                 p = clone.launch_vm_process(type_p=frontend)
-                p.wait_for_completion(60*1000)
+                p.wait_for_completion(60 * 1000)
                 session = clone.create_session()
                 console = session.console
                 guest = console.guest
                 try:
                     guest_session = guest.create_session(username, password,
-                                                         timeout_ms=5*60*1000)
+                                                         timeout_ms=300 * 1000)
                     idle_count = 0
                     timeout = 60
                     while idle_count < 5 and timeout > 0:
-                        act = console.get_device_activity(DeviceType.hard_disk)
-                        if act == DeviceActivity.idle:
+                        act = console.get_device_activity([DeviceType.hard_disk])
+                        if act[0] == DeviceActivity.idle:
                             idle_count += 1
                         time.sleep(0.5)
                         timeout -= 0.5
                     guest_session.close()
                     console.pause()
-                    p = console.take_snapshot('initialised', 'machine pool')
-                    p.wait_for_completion(60*1000)
+                    p, id_p = console.machine.take_snapshot('initialised',
+                                                            'machine pool',
+                                                            True)
+                    p.wait_for_completion(60 * 1000)
                     self._power_down(session)
                 finally:
                     if session.state == SessionState.locked:
@@ -178,16 +186,13 @@ class MachinePool(object):
 
             # Launch our clone
             p = clone.launch_vm_process(type_p=frontend)
-            p.wait_for_completion(60*1000)
+            p.wait_for_completion(60 * 1000)
             session = clone.create_session()
             return session
-    
+
     def release(self, session):
-        "Release a machine session resource."
+        """Release a machine session resource."""
         if session.state != SessionState.locked:
-            return 
+            return
         with self._lock():
             return self._power_down(session)
-
-
-
